import subprocess
import sys
from pathlib import Path
from time import time

from .config import ENVIRONMENT, REPO_BRANCH, REPO_URL
from .yolov5_utils.general import emojis


def install_dependencies():
    t = time()
    print(emojis('Installing YOLOv5 🚀 requirements...'), end=" ")
    s = ['pip', 'install', '-r', 'yolov5/requirements.txt']
    if ENVIRONMENT == 'development':
        subprocess.call(s)
    else:
        subprocess.call(s, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
    print(f'Done ({time() - t:.1f}s)')


def clone_yolo():
    """
    Check that YOLOv5 is on the system and clone if not found
    """
    CWD = str(Path('.').resolve())  # current working directory
    if CWD not in sys.path:
        sys.path.append(CWD)  # add CWD to PATH

    if not Path("./yolov5").is_dir():
        from git import Repo
        try:
            t = time()
            print(emojis(f"Cloning YOLOv5 🚀 from {REPO_URL}..."), end=" ")
            Repo.clone_from(REPO_URL, "yolov5", branch=REPO_BRANCH)  # Temp solution while waiting on pull request
            print(f'Done ({time() - t:.1f}s)')
            install_dependencies()

        except Exception as e:
            print(f"ERROR: Failed to clone YOLOv5: {e}")


class YOLOv5Wrapper:

    @staticmethod
    def export(**kwargs):
        """Calls the YOLOv5 export module"""
        import yolov5.export as _export
        _export.run(**kwargs)

    @staticmethod
    def detect(**kwargs):
        """Calls the YOLOv5 detect module"""
        import yolov5.detect as _detect
        _detect.run(**kwargs)

    @staticmethod
    def train(callback_handler, **kwargs):
        """Calls the YOLOv5 train module"""
        import yolov5.train as _train
        opt = _train.parse_opt(True)
        for k, v in kwargs.items():
            setattr(opt, k, v)

        # _train.run(**kwargs)
        # text_trap = io.StringIO()
        # sys.stdout = text_trap # Trap output
        _train.main(opt, callback_handler)
        # sys.stdout = sys.__stdout__ # Restore output

    @staticmethod
    def new_callback_handler():
        """Returns a YOLOv5 callback handler"""
        from yolov5.utils.callbacks import Callbacks  # assume yolov5/ in cwd
        return Callbacks()



import os

# Global variables
REPO_URL = "https://github.com/ultralytics/yolov5.git"
REPO_BRANCH = "ultralytics/HUB"  # "master"

ENVIRONMENT = os.environ.get("ULTRALYTICS_ENV", "production")
if ENVIRONMENT == 'production':
    HUB_API_ROOT = "https://api.ultralytics.com/"
else:
    HUB_API_ROOT = "http://127.0.0.1:8000/"
    print(f'Connected to development server on {HUB_API_ROOT}')



import requests

from .config import HUB_API_ROOT
from .hub_logger import HUBLogger
from .yolov5_utils.hub_utils import check_dataset_disk_space, smart_post
from .yolov5_wrapper import YOLOv5Wrapper as YOLOv5


class Trainer:

    def __init__(self, model_id, auth):
        self.auth = auth
        self.model = self._get_model(model_id)
        if self.model is not None:
            self._connect_callbacks()

    def _get_model_by_id(self):
        # return a specific model
        return

    def _get_next_model(self):
        # return next model in queue
        return

    def _get_model(self, model_id):
        """
        Returns model from database by id
        """
        api_url = f"{HUB_API_ROOT}/model"
        payload = {"modelId": model_id, **self.auth.get_auth_string()}

        try:
            r = smart_post(api_url, json=payload, thread=False, code=0)
            data = r.json()["data"]
            assert data['data'], 'ERROR: Dataset may still be processing. Please wait a minute and try again.'  # RF fix
            self.model_id = data["id"]
            return data
        except requests.exceptions.ConnectionError as e:
            raise Exception('ERROR: The HUB server is not online. Please try again later.') from e

    def _connect_callbacks(self):
        callback_handler = YOLOv5.new_callback_handler()
        hub_logger = HUBLogger(self.model_id, self.auth)
        callback_handler.register_action("on_pretrain_routine_start", "HUB", hub_logger.on_pretrain_routine_start)
        callback_handler.register_action("on_pretrain_routine_end", "HUB", hub_logger.on_pretrain_routine_end)
        callback_handler.register_action("on_fit_epoch_end", "HUB", hub_logger.on_fit_epoch_end)
        callback_handler.register_action("on_model_save", "HUB", hub_logger.on_model_save)
        callback_handler.register_action("on_train_end", "HUB", hub_logger.on_train_end)
        self.callbacks = callback_handler

    def start(self):
        # Checks
        if not check_dataset_disk_space(self.model['data']):
            return

        # Train
        YOLOv5.train(self.callbacks, **self.model)
